/* packet-diameter.h */

#ifndef __PACKET_DIAMETER_H
#define __PACKET_DIAMETER_H

#include <glib.h>
#include <stdlib.h>
#include <limits.h>
#include <epan/value_string.h>

/* Valid data types */
typedef enum _diam_data_type {
  DIAMETER_NO_TYPE = 0,
  
  /* Base Types */
  DIAMETER_OCTET_STRING = 1,
  DIAMETER_INTEGER32,
  DIAMETER_INTEGER64,
  DIAMETER_UNSIGNED32,
  DIAMETER_UNSIGNED64,
  DIAMETER_FLOAT32,
  DIAMETER_FLOAT64,
  DIAMETER_FLOAT128,
  DIAMETER_GROUPED,

  /* Derived Types */
  DIAMETER_IP_ADDRESS,         /* OctetString */
  DIAMETER_TIME,               /* Integer 32 */
  DIAMETER_UTF8STRING,         /* OctetString */
  DIAMETER_IDENTITY,           /* OctetString */
  DIAMETER_ENUMERATED,         /* Integer 32 */
  DIAMETER_IP_FILTER_RULE,     /* OctetString */
  DIAMETER_QOS_FILTER_RULE,    /* OctetString */
  DIAMETER_MIP_REG_REQ,        /* OctetString */
  DIAMETER_VENDOR_ID,          /* Integer32  */
  DIAMETER_APPLICATION_ID,     /* Integer32  */
  DIAMETER_URI,                /* OctetString */
  DIAMETER_SESSION_ID,          /* OctetString */
  DIAMETER_PUBLIC_ID,			/* OctetString */
  DIAMETER_PRIVATE_ID			/* OctetString */	
} diameter_data_type_t;

typedef struct _vendor_info_t {
	guint32 id;
	gchar* name;
	gchar* longName;
	GHashTable* avps;
	GHashTable* commands;
	GArray* vendors_vs;
	GArray* commands_vs;
	value_string* avp_names;
	value_string* command_names;
} vendor_info_t;

typedef struct _avp_info_t {
	guint32 code;
	diameter_data_type_t type;
	gchar* name;
	gchar* desc;
	const value_string* values;

	int hf;
	int hf_vendor_id;
	int hf_length;
	int hf_flags;
	int hf_flags_vendor_specific;
	int hf_flags_mandatory;
	int hf_flags_protected;
	int hf_flags_reserved3;
	int hf_flags_reserved4;
	int hf_flags_reserved5;
	int hf_flags_reserved6;
	int hf_flags_reserved7;
	gint ett_diameter_avp;
	gint ett_flags;
	gint ett_diameter_avpinfo;	
} avp_info_t;

typedef struct _command_info_t {
	guint32 code;
	vendor_t* vendor;
	gchar* name;
	int hfid;
} command_info_t;

typedef struct _appl_info_t {
	guint32 id;
	gchar *name;
	int hfid;
} appl_info_t;

typedef struct _diameter_group_t {
	GHashTable* avps;
} diameter_group_t;

extern diameter_data_type_t diameter_get_type_id(gchar* typename);
extern guint32 diameter_get_vendor_id(gchar*);

extern add_avp_to_group(avp_info_t*,
						group_t*)

extern vendor_t* diameter_set_vendor(int id,
									 gchar *name,
									 gchar *longName);

extern avp_info_t* diameter_set_avp(guint code,
									const gchar* name,
									diameter_data_type_t type,
									const value_string* values,
									guint vendor_id,
									const gchar* desc);

extern avp_info_t* diameter_set_command(guint32 code,
										gchar* name,
										guint32 vendorId)

extern appl_info_id* diameter_set_application(gchar* name, guint32 id);

extern GString* diameter_parse_dictionary(gchar* fname);

#endif